﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;

namespace Praktikum_2
{
    abstract class Lichtszene
    {
        // Kurzschreibweise für Propertys, bei der ein passendes Feld verdeckt angelegt wird:
        public string Name { get; set; }
        static int nummer;

        // Unschön: Oberflächenelemente und Daten zusammen in einer Klasse
        TabItem tab;

        // Fortgeschritten: eine Factory-Methode
        // Stattdessen könnte man das switch auch im MainWindow machen,
        // also dort, wo diese Methode aufgerufen wird.
        public static Lichtszene ErzeugeVon(TabItem tab)
        {
            switch (tab.Header) // besser mit tab.Tag, weil dann von Übersetzung unabhängig
            {   // Dass ein switch nötig ist, zeigt eigentlich, dass die Polymorphie noch nicht ausgereizt ist.
                // Man könnte z. B. auch mehrere Klassen für die Tabs haben!
                case "Wohnzimmer":
                    return new Wohnzimmerszene(tab);
                case "Küche":
                    return new Küchenszene(tab);
                case "Schlafzimmer":
                    return new Schlafzimmerszene(tab);
                default:
                    return null;
            }
        }

        protected Lichtszene(TabItem tab)
        {
            this.tab = tab;
            Name = "Unbenannt - " + nummer;
            nummer++;
        }

        public virtual void Lade(TabControl tabControl)
        {
            tabControl.SelectedItem = tab;
        }

        // Eine Hilfsfunktion
        public CheckBox FindeCheckBox(string name)
        {
            Grid grid = (Grid)tab.Content;
            foreach (var item in grid.Children) // oder zunächst noch mit einem klassischen for statt des foreach
            {
                if (item is CheckBox)
                {
                    CheckBox cb = (CheckBox)item;
                    if(cb.Content.ToString() == name) // besser mit Tag statt Content
                    {
                        return cb;
                    }
                }
            }
            return null;
        }
    }

    class Wohnzimmerszene : Lichtszene
    {
        // Kurzschreibweise, bei der ein passendes Feld verdeckt angelegt wird:
        public bool Deckenleuchte { get; private set; }
        public bool Leseleuchte { get; private set; }
        public bool Schrankleuchte { get; private set; }
        public bool Tischleuchte { get; private set; }

        public Wohnzimmerszene(TabItem tab)
            : base(tab)
        {
            Deckenleuchte = FindeCheckBox("Deckenleuchte").IsChecked.Value;
            Leseleuchte = FindeCheckBox("Leseleuchte").IsChecked.Value;
            Schrankleuchte = FindeCheckBox("Schrankleuchte").IsChecked.Value;
            Tischleuchte = FindeCheckBox("Tischleuchte").IsChecked.Value;
        }

        public override void Lade(TabControl tabControl)
        {   
            FindeCheckBox("Deckenleuchte").IsChecked = Deckenleuchte;
            FindeCheckBox("Leseleuchte").IsChecked = Leseleuchte;
            FindeCheckBox("Schrankleuchte").IsChecked = Schrankleuchte;
            FindeCheckBox("Tischleuchte").IsChecked = Tischleuchte;
            base.Lade(tabControl);
        }

        public override string ToString()
        {
            return "Wohnzimmer: " + Name;
        }
    }

    class Küchenszene : Lichtszene
    {
        public bool Deckenleuchte { get; private set; }
        public bool UnterSchrank { get; private set; }

        public Küchenszene(TabItem tab)
            : base(tab)
        {
            Deckenleuchte = FindeCheckBox("Deckenleuchte").IsChecked.Value;
            UnterSchrank = FindeCheckBox("Unter Schrank").IsChecked.Value;
        }

        public override void Lade(TabControl tabControl)
        {
            FindeCheckBox("Deckenleuchte").IsChecked = Deckenleuchte;
            FindeCheckBox("Unter Schrank").IsChecked = UnterSchrank;
            base.Lade(tabControl);
        }

        public override string ToString()
        {
            return "Küche: " + Name;
        }
    }

    class Schlafzimmerszene : Lichtszene
    {
        public bool Deckenleuchte { get; private set; }
        public bool Nachttischleuchte { get; private set; }
        public bool Schreibtischleuchte { get; private set; }


        public Schlafzimmerszene(TabItem tab)
            : base(tab)
        {
            Deckenleuchte = FindeCheckBox("Deckenleuchte").IsChecked.Value;
            Nachttischleuchte = FindeCheckBox("Nachttischleuchte").IsChecked.Value;
            Schreibtischleuchte = FindeCheckBox("Schreibtischleuchte").IsChecked.Value;
        }

        public override void Lade(TabControl tabControl)
        {
            FindeCheckBox("Deckenleuchte").IsChecked = Deckenleuchte;
            FindeCheckBox("Nachttischleuchte").IsChecked = Nachttischleuchte;
            FindeCheckBox("Schreibtischleuchte").IsChecked = Schreibtischleuchte;
            base.Lade(tabControl);
        }

        public override string ToString()
        {
            return "Schlafzimmer: " + Name;
        }
    }
}
